
/* Chrysalide - Outil d'analyse de fichiers binaires
 * block.c - encadrement des instructions par blocs
 *
 * Copyright (C) 2012-2013 Cyrille Bagard
 *
 *  This file is part of Chrysalide.
 *
 *  OpenIDA is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  OpenIDA is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 */


#include "block.h"


#include "block-int.h"



/* Initialise la classe des blocs d'instructions. */
static void g_instr_block_class_init(GInstrBlockClass *);

/* Initialise un bloc d'instructions. */
static void g_instr_block_init(GInstrBlock *);

/* Supprime toutes les références externes. */
static void g_instr_block_dispose(GInstrBlock *);

/* Procède à la libération totale de la mémoire. */
static void g_instr_block_finalize(GInstrBlock *);



/* Indique le type défini pour un bloc d'instructions. */
G_DEFINE_TYPE(GInstrBlock, g_instr_block, G_TYPE_OBJECT);


/******************************************************************************
*                                                                             *
*  Paramètres  : class = classe à initialiser.                                *
*                                                                             *
*  Description : Initialise la classe des blocs d'instructions.               *
*                                                                             *
*  Retour      : -                                                            *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

static void g_instr_block_class_init(GInstrBlockClass *class)
{
    GObjectClass *object;                   /* Autre version de la classe  */

    object = G_OBJECT_CLASS(class);

    object->dispose = (GObjectFinalizeFunc/* ! */)g_instr_block_dispose;
    object->finalize = (GObjectFinalizeFunc)g_instr_block_finalize;

}


/******************************************************************************
*                                                                             *
*  Paramètres  : block = instance à initialiser.                              *
*                                                                             *
*  Description : Initialise un bloc d'instructions.                           *
*                                                                             *
*  Retour      : -                                                            *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

static void g_instr_block_init(GInstrBlock *block)
{

}


/******************************************************************************
*                                                                             *
*  Paramètres  : block = instance d'objet GLib à traiter.                     *
*                                                                             *
*  Description : Supprime toutes les références externes.                     *
*                                                                             *
*  Retour      : -                                                            *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

static void g_instr_block_dispose(GInstrBlock *block)
{
    if (block->links_block != NULL)
        g_object_unref(G_OBJECT(block->links_block));

    G_OBJECT_CLASS(g_instr_block_parent_class)->dispose(G_OBJECT(block));

}


/******************************************************************************
*                                                                             *
*  Paramètres  : block = instance d'objet GLib à traiter.                     *
*                                                                             *
*  Description : Procède à la libération totale de la mémoire.                *
*                                                                             *
*  Retour      : -                                                            *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

static void g_instr_block_finalize(GInstrBlock *block)
{
    G_OBJECT_CLASS(g_instr_block_parent_class)->finalize(G_OBJECT(block));

}


/******************************************************************************
*                                                                             *
*  Paramètres  : block = bloc de départ des recherches.                       *
*                addr  = ensemble de blocs à parcourir.                       *
*                final = indique si la cible ou le conteneur est renvoyée.    *
*                                                                             *
*  Description : Recherche le bloc contenant une adresse donnée.              *
*                                                                             *
*  Retour      : bloc basique trouvé ou NULL en cas d'échec.                  *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

GInstrBlock *g_instr_block_find_by_addr(const GInstrBlock *block, const vmpa2t *addr, bool final)
{
    return G_INSTR_BLOCK_GET_CLASS(block)->find_by_addr(block, addr, final);

}


/******************************************************************************
*                                                                             *
*  Paramètres  : block    = bloc d'instructions démarrant la visite.          *
*                callback = ensemble de blocs à parcourir.                    *
*                data     = donnée utilisateur à associer au parcours.        *
*                                                                             *
*  Description : Parcourt tous les blocs d'instructions dans un ordre donné.  *
*                                                                             *
*  Retour      : true si le parcours a été jusqu'à son terme, false sinon.    *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

bool g_instr_block_visit(GInstrBlock *block, instr_block_visitor_cb callback, void *data)
{
    return G_INSTR_BLOCK_GET_CLASS(block)->visit_blocks(block, callback, data);

}


/******************************************************************************
*                                                                             *
*  Paramètres  : block = bloc d'instructions à consulter.                     *
*                list  = ensemble de blocs à compléter. [OUT]                 *
*                count = nombre de blocs au total. [OUT]                      *
*                                                                             *
*  Description : Etablit la liste de tous les blocs présents.                 *
*                                                                             *
*  Retour      : -                                                            *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

void g_instr_block_list_all_blocks(const GInstrBlock *block, GInstrBlock ***list, size_t *count)
{
    G_INSTR_BLOCK_GET_CLASS(block)->list_blocks(block, list, count);

}


/******************************************************************************
*                                                                             *
*  Paramètres  : block = bloc d'instructions à consulter.                     *
*                list  = ensemble de blocs à compléter. [OUT]                 *
*                count = nombre de blocs au total. [OUT]                      *
*                                                                             *
*  Description : Etablit la liste de tous les blocs terminaux.                *
*                                                                             *
*  Retour      : -                                                            *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

void g_instr_block_list_leafs_blocks(const GInstrBlock *block, GInstrBlock ***list, size_t *count)
{
    G_INSTR_BLOCK_GET_CLASS(block)->list_leafs(block, list, count);

}


/******************************************************************************
*                                                                             *
*  Paramètres  : block = bloc d'instructions à mettre à jour.                 *
*                links = bloc contenant les blocs liés au bloc.               *
*                                                                             *
*  Description : Définit l'ensemble contenant les blocs liés.                 *
*                                                                             *
*  Retour      : -                                                            *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

void g_instr_block_set_links_block(GInstrBlock *block, GInstrBlock *links)
{
    if (block->links_block != NULL)
        g_object_unref(G_OBJECT(block->links_block));

    g_object_ref(G_OBJECT(links));
    block->links_block = links;

}


/******************************************************************************
*                                                                             *
*  Paramètres  : block = bloc d'instructions à mettre à jour.                 *
*                                                                             *
*  Description : Fournit l'ensemble contenant les blocs liés.                 *
*                                                                             *
*  Retour      : Bloc contenant les blocs liés au bloc.                       *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

GInstrBlock *g_instr_block_get_links_block(const GInstrBlock *block)
{
    return block->links_block;

}
